/*****************************
File:      RemoteSFBool.h
Language:   C++ (header)
Project:    H3DNetworkingUtils
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in
compliance with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS"
basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
License for the specific language governing rights and limitations
under the License.

The Original Code is H3DNetworkingUtils v1.0.

The Initial Developer of the Original Code is CSIRO.
Portions created by the Initial Developer are Copyright (C) 1009 CSIRO. All Rights Reserved.
Contributor(s):
    Chris Gunn  <Chris.Gunn@csiro.au> <ChrisJGunn@gmail.com>
***************************/

/// \file RemoteSFBool.h
/// \brief Header file for RemoteSFBool, sends and receives a boolean value.

#ifndef RemoteSFBool_H
#define RemoteSFBool_H


#include "H3DNetworkingUtils/RemoteSF.h"
#include <H3D/SFBool.h>

namespace H3DNetworkingUtils {
   
/// \class RemoteSFBool
/// The RemoteSFBool class provides a remote single boolean field.  It inherits most of its funtionality from its parent
/// class.
/// This class adds a X3D database and the details of how to read and write
/// the field data. \n
/// Use the 'toBeSent' and 'received' fields of the base class.  \n
/// <b>Examples:</b>
///   - <a href="../../examples/RemoteSFTestServer.x3d">RemoteMFTestServer.x3d</a>
///   - <a href="../../examples/RemoteSFTestClient.x3d">RemoteMFTestClient.x3d</a>

class H3D_NETWORKING_UTILS_DLL_SPEC RemoteSFBool : public RemoteSF<H3D::SFBool> {
public:
   /// Constructor
   RemoteSFBool();
  
   /// X3D interface
   static H3D::H3DNodeDatabase database;

   /// This is a debug function only \n  To be used to simulate receiving a remote
   /// connection change of value, when there is no remote connection.
   void setValue(bool const & val);

protected:
   /// Read the parts of a fields value into the received field.
   virtual void readValue();

   /// Write the correct parts of the value.
   virtual void writeValue(bool const & val) {writeBool(val);}
};

}

#endif
